#include "ble_client.h"
#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/event_groups.h"
#include "esp_event.h"
#include "nvs_flash.h"
#include "esp_log.h"
#include "esp_nimble_hci.h"
#include "nimble/nimble_port.h"
#include "nimble/nimble_port_freertos.h"
#include "host/ble_hs.h"
#include "services/gap/ble_svc_gap.h"
#include "wit.h"
#include "acc.h"
#include "ram.h"
#include "flash.h"
#include "rtc.h"
#include "if_ble_config.h"
#include "if_gw.h"
#include "if_apkg_code.h"

#define tag  "ble"

#define WIT_BLE_TASK_STK      (configMINIMAL_STACK_SIZE * 2)
#define WIT_BLE_TASK_PRIORITY (tskIDLE_PRIORITY + 3)

uint8_t ble_addr_type;

static char  bin_buf[512] = {0};
static unsigned long  g_bDevTime;
static bool bIsAdv = true;

static  ble_uuid128_t sensor_server = {
    .u.type = BLE_UUID_TYPE_128,
    .value = {0x4d, 0x55, 0x49, 0x54, 0x49, 0x57, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0x01, 0xfe, 0x43, 0x33},
};
static  ble_uuid128_t sensor_char = {
    .u.type = BLE_UUID_TYPE_128,                                        
    .value = {0x4d, 0x55, 0x49, 0x54, 0x49, 0x57, 0x00, 0x80, 0x00, 0x10, 0x00, 0x00, 0xf0, 0xff, 0x43, 0x33},
};

sSenInfo gSenInfo;

static void ble_app_scan(void);
static void blecent_on_disc_complete(const struct peer *peer, int status, void *arg);
static void blecent_connect_if_interesting(const struct ble_gap_disc_desc *disc);
static bool __help_ble_indicate(uint16_t conn_handle, uint16_t val_handle, void *buf, size_t len);
static bool ble_operate(int index, void *data, uint16_t size, uint16_t handle, action_class ac);

static void ble_task(void *pvParameter);

// BLE event handling
static int ble_gap_event(struct ble_gap_event *event, void *arg)
{
    struct ble_gap_conn_desc desc;
    struct ble_hs_adv_fields fields;
    int rc;
    char name[48];

    switch (event->type) {
    case BLE_GAP_EVENT_DISC:
        rc = ble_hs_adv_parse_fields(&fields, event->disc.data, event->disc.length_data);
        if (rc != 0) {
            return 0;
        }
        print_adv_fields(&fields);
        memcpy(name, fields.name, fields.name_len);   
        sSenTable     *pxSenTable = NULL;
        int            iIndex = 0;
        // 判断ble是否连接
        for(int i = 0; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ ) 
        {
            pxSenTable = getSensorTable(i);
            if(pxSenTable == NULL)
                return 0;
            if(pxSenTable->ulUseEnable == 1) 
            {
                if(strstr(name, pxSenTable->cName) != NULL  && fields.mfg_data[0] == 'G') 
                {
                    ESP_LOGI("ble name","%s", pxSenTable->cName);
                    ESP_LOGD("addr","%ld", pxSenTable->ulAddr);
                    bIsAdv = false;
                    iIndex = (int)(pxSenTable->ulAddr - 1);  
                    if(iIndex < 0)
                        return 0;
                    if(gSenInfo.xDevInfo[iIndex].bIsConnect == false && gSenInfo.xDevInfo[iIndex].cComplete == 0)
                    {
                        if(fields.mfg_data[1] == 'E')
                            gSenInfo.xDevInfo[iIndex].senCode = 1;
                        else
                            gSenInfo.xDevInfo[iIndex].senCode = 0;
                        gSenInfo.xDevInfo[iIndex].addr = pxSenTable->ulAddr;
                        gSenInfo.xDevInfo[iIndex].ulVirAddr = pxSenTable->ulVirAddr;
                        gSenInfo.xDevInfo[iIndex].group = pxSenTable->ulGroup;
                        gSenInfo.xDevInfo[iIndex].bIsFind = true;
                        blecent_connect_if_interesting(&event->disc); 
                        g_bDevTime = xTaskGetTickCount();
                    } 
                }
            } 
        }
        return 0;

    case BLE_GAP_EVENT_CONNECT:
        bool bConnectFlag = false;
        if (event->connect.status == 0) {
            ESP_LOGD(tag, "Connection established ");
            ble_gap_conn_find(event->connect.conn_handle, &desc);
            rc = peer_add(event->connect.conn_handle);     /* Remember peer. */
            if (rc != 0) {
                ble_gap_terminate(event->connect.conn_handle, BLE_ERR_REM_USER_CONN_TERM);
                ESP_LOGD(tag, "Failed to add peer; rc=%d\n", rc);
                return 0;
            }
            for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ ) 
            {
                if(gSenInfo.xDevInfo[i].bIsFind == true && gSenInfo.xDevInfo[i].bIsConnect == false)
                {
                    bConnectFlag = true;
                    gSenInfo.xDevInfo[i].peer =  peer_find(event->connect.conn_handle);
                    gSenInfo.xDevInfo[i].handle = event->connect.conn_handle;
                    gSenInfo.xDevInfo[i].bIsFind = false;
                }
            }
            if(bConnectFlag == false)      
            {
                peer_delete(event->connect.conn_handle); 
                ble_gap_terminate(event->connect.conn_handle, BLE_ERR_REM_USER_CONN_TERM);
                return 0;
            } 

            rc = peer_disc_all(event->connect.conn_handle, blecent_on_disc_complete, NULL);       
        } else {
            ble_app_scan();
        }
        return 0;

    case BLE_GAP_EVENT_DISCONNECT:
        bool bStatus = false;
        ESP_LOGD(tag, "disconnect; reason=%d", event->disconnect.reason);
        for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM ; i++ )
        {
            if(event->disconnect.conn.conn_handle == gSenInfo.xDevInfo[i].handle) 
            {
                bStatus = true;
                peer_delete(event->disconnect.conn.conn_handle);  
                ble_clear_info_from_index(i);
            }
        } 
        if(bStatus == false)
        {
            peer_delete(event->disconnect.conn.conn_handle); 
            ble_gap_terminate(event->disconnect.conn.conn_handle, BLE_ERR_REM_USER_CONN_TERM); 
        } 
        ble_app_scan();   
        return 0;

    case BLE_GAP_EVENT_DISC_COMPLETE:
        ESP_LOGD(tag, "discovery complete; reason=%d\n", event->disc_complete.reason);
        return 0;

    case BLE_GAP_EVENT_ENC_CHANGE:
        ESP_LOGD(tag, "encryption change event; status=%d ", event->enc_change.status);
        rc = ble_gap_conn_find(event->enc_change.conn_handle, &desc);
        assert(rc == 0);
        print_conn_desc(&desc);
        return 0;

    case BLE_GAP_EVENT_NOTIFY_RX:
        ESP_LOGD(tag, "received %s; conn_handle=%d attr_handle=%d " "attr_len=%d\n", event->notify_rx.indication ? "indication" : "notification",
                event->notify_rx.conn_handle, event->notify_rx.attr_handle, OS_MBUF_PKTLEN(event->notify_rx.om));
        for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ ) {
            if(event->notify_rx.conn_handle == gSenInfo.xDevInfo[i].handle){
                ble_operate(i, event->notify_rx.om->om_data, OS_MBUF_PKTLEN(event->notify_rx.om), event->notify_rx.attr_handle, OPERATE_INDICATE);
            }
        }
        return 0;

    case BLE_GAP_EVENT_MTU:
        ESP_LOGD(tag, "mtu update event; conn_handle=%d cid=%d mtu=%d\n", event->mtu.conn_handle, event->mtu.channel_id, event->mtu.value);

        return 0;

    case BLE_GAP_EVENT_CONN_UPDATE:
        ESP_LOGD("ble","BLE_GAP_EVENT_CONN_UPDATE");
        ble_att_set_preferred_mtu(498);
        ble_gattc_exchange_mtu(event->connect.conn_handle, NULL, NULL);
        return 0;

    case BLE_GAP_EVENT_REPEAT_PAIRING:
        rc = ble_gap_conn_find(event->repeat_pairing.conn_handle, &desc);
        assert(rc == 0);
        ble_store_util_delete_peer(&desc.peer_id_addr);
        return BLE_GAP_REPEAT_PAIRING_RETRY;
#if CONFIG_EXAMPLE_EXTENDED_ADV
    case BLE_GAP_EVENT_EXT_DISC:
        ext_print_adv_report(&event->disc);
        // blecent_connect_if_interesting(&event->disc);
        return 0;
#endif
    default:
        return 0;
    }
}

static void ble_app_scan(void)
{
    printf("Start scanning ...\n");
    int rc;
    struct ble_gap_disc_params disc_params;
    disc_params.filter_duplicates = 1;
    disc_params.passive = 0;
    // disc_params.itvl =  BLE_GAP_SCAN_FAST_INTERVAL_MIN;
    // disc_params.window = BLE_GAP_SCAN_FAST_WINDOW;
    disc_params.itvl =  BLE_GAP_SCAN_ITVL_MS(20);
    disc_params.window = BLE_GAP_SCAN_WIN_MS(20);
    disc_params.filter_policy = 0;
    disc_params.limited = 0;
    rc = ble_gap_disc(ble_addr_type, BLE_HS_FOREVER, &disc_params, ble_gap_event, NULL);
    if (rc != 0) {
        ESP_LOGD(tag, "Error initiating GAP discovery procedure; rc=%d\n", rc);
    }
    bIsAdv = true;
}

void ble_app_on_sync(void)
{
    ble_hs_id_infer_auto(0, &ble_addr_type); // Determines the best address type automatically
    ble_app_scan();                          
}

void host_task(void *param)
{
    nimble_port_run(); // This function will return only when nimble_port_stop() is executed
}

bool ble_client_init(void)
{
    nvs_flash_init();                               // 1 - Initialize NVS flash using
    wcp_init(APKG_MASTER);
    ble_clear_dev_info();
    nimble_port_init();                             // 3 - Initialize the controller stack
    peer_init(MYNEWT_VAL(BLE_MAX_CONNECTIONS), 64, 64, 64);   /* Initialize data structures to track connected peers. */
    ble_svc_gap_device_name_set("Witium-Client");             // 4 - Set device name characteristic
    ble_svc_gap_init();                             // 4 - Initialize GAP service
    ble_hs_cfg.sync_cb = ble_app_on_sync;           // 5 - Set application
    nimble_port_freertos_init(host_task);           // 6 - Set infinite task
    xTaskCreate(ble_task, "ble task", WIT_BLE_TASK_STK, NULL, WIT_BLE_TASK_PRIORITY, NULL);
    return true;
}

static void ble_task(void *pvParameter)
{   
    while(1)
    {
        for(int i = 0; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++) 
        {
            if(gSenInfo.xDevInfo[i].bIsConnect == true)
            {
                ble_gap_conn_rssi(gSenInfo.xDevInfo[i].handle, (int8_t *)&gSenInfo.xDevInfo[i].rssi);
            }
        } 
        if(abs((long long)xTaskGetTickCount() - (long long)g_bDevTime) > 3 * 30 * 1000.0f)
        {
            if(bIsAdv == false)
            {
                ble_gap_disc_cancel();
                ble_app_scan();         
            }
        }
        vTaskDelay(5 / portTICK_PERIOD_MS);
    }
}
static void blecent_on_disc_complete(const struct peer *peer, int status, void *arg)
{
    bool bStatus = false;
    if (status != 0) {
        ESP_LOGD(tag, "Error: Service discovery failed; status=%d " "conn_handle=%d\n", status, peer->conn_handle);   
        peer_delete(peer->conn_handle);        
        ble_gap_terminate(peer->conn_handle, BLE_ERR_REM_USER_CONN_TERM);       
        return;
    }
    ESP_LOGD(tag, "Service discovery complete; status=%d " "conn_handle=%d\n", status, peer->conn_handle);
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ ) 
    {
        if(peer->conn_handle == gSenInfo.xDevInfo[i].handle)
        {
            gSenInfo.xDevInfo[i].bIsConnect = true; 
            bStatus = true;
            ESP_LOGD("wake check handle","%d", peer->conn_handle);
            if(ble_operate(i, NULL, 0, gSenInfo.xDevInfo[i].handle, OPEN_CONNECT) == false)
            {
                peer_delete(peer->conn_handle);   
                ble_gap_terminate(gSenInfo.xDevInfo[i].handle, BLE_ERR_REM_USER_CONN_TERM);
                ESP_LOGD(tag, "sleep error");    
            }
        }
    } 
    if(bStatus == false)
    {
        peer_delete(peer->conn_handle);   
        ble_gap_terminate(peer->conn_handle, BLE_ERR_REM_USER_CONN_TERM);
    }
}

static void blecent_connect_if_interesting(const struct ble_gap_disc_desc *disc)
{
    uint8_t own_addr_type;
    int rc;  

    rc = ble_gap_disc_cancel();
    if (rc != 0) {
        ESP_LOGD(tag, "Failed to cancel scan; rc=%d\n", rc);
        return;
    }
    rc = ble_hs_id_infer_auto(0, &own_addr_type);
    if (rc != 0) {
        ESP_LOGD(tag, "error determining address type; rc=%d\n", rc);
        return;
    }

    ESP_LOGD(tag, "connect ");
    rc = ble_gap_connect(own_addr_type, &disc->addr, 10000, NULL, ble_gap_event, NULL);
    if (rc != 0) {
        ESP_LOGD(tag, "Error: Failed to connect to device; addr_type=%d " "addr=%s\n; rc=%d", disc->addr.type, addr_str(disc->addr.val), rc);
        return;
    }
}

static bool ble_operate(int index, void *data, uint16_t size, uint16_t handle, action_class ac)
{
    switch(ac)
    {
        case OPERATE_INDICATE: 
            if(gSenInfo.xDevInfo[index].sensor_chr != NULL && handle == gSenInfo.xDevInfo[index].sensor_chr->chr.val_handle) 
            {
                wcp_data_migration(index, SENSOR_CHR_WRITE_HANDLER, (unsigned char *)data, size);
            }
        break;
        case OPEN_CONNECT:
            if(gSenInfo.xDevInfo[index].peer != NULL)
            {
                gSenInfo.xDevInfo[index].sensor_chr = peer_chr_find_uuid(gSenInfo.xDevInfo[index].peer, (ble_uuid_t *)&sensor_server, (ble_uuid_t *)&sensor_char);
            }
            if(gSenInfo.xDevInfo[index].sensor_chr != NULL)
            {
                ble_gattc_indicate(gSenInfo.xDevInfo[index].peer->conn_handle, gSenInfo.xDevInfo[index].sensor_chr->chr.val_handle);
            }  
            if(gSenInfo.xDevInfo[index].sensor_chr == NULL || gSenInfo.xDevInfo[index].peer == NULL) 
            {
                printf("重连...............\r\n");
                ble_gap_terminate(handle, BLE_ERR_REM_USER_CONN_TERM);
            } 
            if(if_check_wakeup_time(gSenInfo.xDevInfo[index].group, gSenInfo.xDevInfo[index].addr, index) == false)
            {
                return false;
            }    
        break;
        default:
            break;   
    }
     return true;
}

unsigned long getBLEConnectNum(void)
{
    unsigned long ulExistNum = 0;
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++ ) {
        if(gSenInfo.xDevInfo[i].bIsConnect == true) {
            ulExistNum++;        
        }
    }
    return ulExistNum;
}


static bool __help_ble_indicate(uint16_t conn_handle, uint16_t val_handle, void *buf, size_t len)
{
    struct os_mbuf *om = ble_hs_mbuf_from_flat(buf, len);
    if (om == NULL)
    {
        ESP_LOGW("ble", "Error in allocating memory");
        return false;
    }
    int rc = ble_gattc_indicate_custom(conn_handle, val_handle, om);
    if (rc != 0)
    {
        ESP_LOGW("ble", "Error in sending notification");
        return false;
    }
    return true;
}

bool ble_sensor_write(int index, void *buf, size_t len)
{   
    int rc = 0;
    if(gSenInfo.xDevInfo[index].sensor_chr == NULL || gSenInfo.xDevInfo[index].addr == 0 || gSenInfo.xDevInfo[index].bIsConnect == false)
    {
        return false;
    }
    rc = ble_gattc_write_flat(gSenInfo.xDevInfo[index].peer->conn_handle, gSenInfo.xDevInfo[index].sensor_chr->chr.val_handle, buf, len, NULL, NULL);   
    if(rc == 0){
        return true;
    }
    return false;
}

bool ble_sensor_indicate(int index, void *buf, size_t len)
{
    if(gSenInfo.xDevInfo[index].sensor_chr == NULL || gSenInfo.xDevInfo[index].bIsConnect == false)
    {
        return false;
    }
    return __help_ble_indicate(gSenInfo.xDevInfo[index].peer->conn_handle, gSenInfo.xDevInfo[index].sensor_chr->chr.val_handle, buf, len);
}

bool send_update_bin(int index)
{
    unsigned long ulSize = 0;
    unsigned int sz = 0;
    unsigned int address = SENSOR_UPDATE_BIN_APKG_ADDR;
    unsigned char flag = 0xff;
    ulSize = getBleBinSize();
    esp_err_t ret;
    printf("ulSize = %ld\r\n", ulSize);
    // 发启更新
     eWCPWriteDataAddress(index, SENSOR_CHR_WRITE_HANDLER, SENSOR_UPDATE_START_APKG_ADDR, 1, &flag);
    // 更新
    for(size_t i = 0; i < ulSize; i += 468) 
    {
        sz = (ulSize - i) > 468  ? 468 : (ulSize - i);
        ret = read_ble_bin(bin_buf, i, sz);
        if(ret != ESP_OK)
            return false;
        if(eWCPWriteDataAddress(index, SENSOR_CHR_WRITE_HANDLER, address, sz, &bin_buf) != WCP_OK)
            return false;
        address += sz;
        vTaskDelay(20 / portTICK_PERIOD_MS);
    }
    // 结束更新
    eWCPWriteDataAddress(index, SENSOR_CHR_WRITE_HANDLER, SENSOR_UPDATE_END_APKG_ADDR, 1, &flag);
    return true;
}

void ble_clear_info_from_index(int index)
{
    if(index >= WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM)
        return;
    
    //gSenInfo.xDevInfo[index].addr = 0;
    gSenInfo.xDevInfo[index].group = 0;
    gSenInfo.xDevInfo[index].handle = 0xFFFF;
    // gSenInfo.xDevInfo[index].bIsFind = false;
    gSenInfo.xDevInfo[index].bIsConnect = false; 
    gSenInfo.xDevInfo[index].peer = NULL;
    gSenInfo.xDevInfo[index].sensor_chr = NULL;
}

void ble_clear_dev_info(void)
{
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++)
    {
        gSenInfo.xDevInfo[i].cComplete = 0;
        gSenInfo.xDevInfo[i].bIsFind = false;
        ble_clear_info_from_index(i);
    }
    gSenInfo.ulConnnectBit = 0;
}

void ble_clear_dev_complete(void)
{
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++)
    {
        gSenInfo.xDevInfo[i].cComplete = 0;
    }
}

sDevInfo *ble_find_dev_from_addr(unsigned short usAddr)
{
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++)
    {
        if(usAddr == gSenInfo.xDevInfo[i].addr)
            return &gSenInfo.xDevInfo[i];
    }
    return NULL;
}

sDevInfo *ble_find_dev_from_group(unsigned short usGroup)
{
    for(int i = 0 ; i < WIT_SENSOR_MAX_CONNECT_NUM * WIT_SENSOR_GROUP_MAX_NUM; i++)
    {
        if(usGroup == gSenInfo.xDevInfo[i].group)
            return &gSenInfo.xDevInfo[i];
    }
    return NULL;
}