/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "mqtt_update.h"
#include "esp_ota_ops.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/timers.h"
#include "esp_log.h"

/* Private macros ------------------------------------------------------------*/
#define MQTT_UPDATE_PACKAGE_MIN_SIZE    6
#define MQTT_UPDATE_PER_SIZE            4096
#define LOG_TAG                         "mqtt_update"

/* Private types -------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void Update_GWTimeoutHandler(TimerHandle_t xTimer);
/* Private variables ---------------------------------------------------------*/
static bool g_bGWUpdate = false;
static Update_ResponseStatus g_eUpdateResponse = Response_Null;
static unsigned short g_ucTotalUpdateSize = 0;
static unsigned short g_ucCurrentPackageID = 0;
static TimerHandle_t g_xGWUpdateTimer;

static const esp_partition_t *update_partition = NULL;
static esp_ota_handle_t update_handle;
static unsigned int gBCurrent = 0;

void mqtt_update_init(void)
{
	// check which partition is running
	const esp_partition_t *partition = esp_ota_get_running_partition();	
	
	switch (partition->address) {
    case 0x00010000:
      ESP_LOGD(LOG_TAG, "Running partition: factory");
      break;
    case 0x00100000:
      ESP_LOGD(LOG_TAG, "Running partition: ota_0");
      break;
    case 0x00200000:
      ESP_LOGD(LOG_TAG, "Running partition: ota_1");
      break;
    default:
      ESP_LOGD(LOG_TAG, "Running partition: unknown");
      break;
  }
}
/**
 * @brief  GW Update
 * @param
 * @retval None
 */
void Update_GatewayHandler(char *pcGWUpdateString, unsigned long ulLen)
{
	unsigned long ulReceivedCheckSum = 0;
	unsigned long ulCalculatedCheckSum = 0;
	char *pcDataPointer = NULL;
	unsigned long ulDataLen = 0;
	int i;
    esp_err_t err;

	if (ulLen < MQTT_UPDATE_PACKAGE_MIN_SIZE)
		return;
    
	// get current package id
	g_ucCurrentPackageID = ((short)pcGWUpdateString[1]) |((short)pcGWUpdateString[0] << 8);
	// get package check sum
	ulReceivedCheckSum = (unsigned long)pcGWUpdateString[ulLen - 4] << 24 | (unsigned long)pcGWUpdateString[ulLen - 3] << 16 |
						 (unsigned long)pcGWUpdateString[ulLen - 2] << 8 | (unsigned long)pcGWUpdateString[ulLen - 1];
    // esp_log_buffer_hex("updata", pcGWUpdateString, ulLen);	
	// calculate check sum
	for (i = 0; i < ulLen - 4; i++)
	{
		ulCalculatedCheckSum += pcGWUpdateString[i];
	}
	// compare
	if (ulCalculatedCheckSum == ulReceivedCheckSum)
	{
		pcDataPointer = &pcGWUpdateString[2];
		ulDataLen = ulLen - MQTT_UPDATE_PACKAGE_MIN_SIZE;
		if (g_ucCurrentPackageID == 0)
		{
			gBCurrent = 0;
			esp_ota_end(update_handle);
			// check flag
			if (ulDataLen - 2 == strlen("Witium_Update") && memcmp(pcDataPointer, "Witium_Update", ulDataLen - 2) == 0)
			{
				// Timer Init
				if (g_xGWUpdateTimer == NULL)
					g_xGWUpdateTimer = xTimerCreate("UpdateTimer", UPDATE_TIMEOUT, pdFALSE, NULL, Update_GWTimeoutHandler);

				// Timer start
				if (g_xGWUpdateTimer != NULL)
					xTimerStart(g_xGWUpdateTimer, 1000);

				// get total package
				g_ucTotalUpdateSize = ((short)pcGWUpdateString[ulLen - 5]) |((short)pcGWUpdateString[ulLen - 6] << 8);
				ESP_LOGD(LOG_TAG,"%d", (int)g_ucTotalUpdateSize);
				g_bGWUpdate = true;
				g_eUpdateResponse = Response_Success;
                
				update_partition = esp_ota_get_next_update_partition(NULL);
				ESP_LOGD(LOG_TAG, "flash start = %x!", (int)update_partition->address);
				esp_ota_begin(update_partition, OTA_WITH_SEQUENTIAL_WRITES, &update_handle);
			}
		}
		else if (g_bGWUpdate == true)
		{
			if (g_ucCurrentPackageID < g_ucTotalUpdateSize)
			{
				if (ulDataLen == MQTT_UPDATE_PER_SIZE)
				{
					// restart timer
					if (g_xGWUpdateTimer != NULL)
						xTimerStart(g_xGWUpdateTimer, 1000);
					// write data
					if(gBCurrent == g_ucCurrentPackageID) {
						err = esp_ota_write(update_handle, (const void *)pcDataPointer, ulDataLen);
						if (err != ESP_OK) {
							ESP_LOGD(LOG_TAG, "esp_ota_write failed (%s)!", esp_err_to_name(err));
						}
						g_eUpdateResponse = Response_Success;
					} else {
						gBCurrent = 0;
						g_bGWUpdate = false;
						esp_ota_end(update_handle);
						g_eUpdateResponse = Response_Error;
					}
				}
			}
			else if (g_ucCurrentPackageID == g_ucTotalUpdateSize)
			{
				// stop timer
				if (g_xGWUpdateTimer != NULL)
					xTimerStop(g_xGWUpdateTimer, 1000);
				// write data
				if(gBCurrent == g_ucCurrentPackageID) {
					gBCurrent = 0;
					err = esp_ota_write(update_handle, (const void *)pcDataPointer, ulDataLen);
					if (err != ESP_OK) {
						ESP_LOGD(LOG_TAG, "esp_ota_write failed (%s)!", esp_err_to_name(err));
					}
					g_eUpdateResponse = Response_Reset;
				} else{
					gBCurrent = 0;
					g_bGWUpdate = false;
					esp_ota_end(update_handle);
					g_eUpdateResponse = Response_Error;
				}
			}
		}
	}

	if (g_eUpdateResponse == Response_Null)
		g_eUpdateResponse = Response_Error;
	gBCurrent++;
}

/**
 * @brief  Update get status
 * @param
 * @retval None
 */
bool Update_GetStatus()
{
	return (bool)(g_bGWUpdate);
}

/**
 * @brief  Update get gw status
 * @param
 * @retval None
 */
bool Update_GetGWStatus()
{
	return g_bGWUpdate;
}

/**
 * @brief  Update poll
 * @param
 * @retval None
 */
Update_ResponseStatus Update_Poll(unsigned char *pucCurrentPackageID)
{
	Update_ResponseStatus eStatus = Response_Null;
    esp_err_t err;
	if (g_eUpdateResponse != Response_Null)
	{
		switch (g_eUpdateResponse)
		{
		case Response_Reset:
			 // end the OTA and start validation
			err = esp_ota_end(update_handle);
			if (err == ESP_OK) {
				g_bGWUpdate = false;
				err = esp_ota_set_boot_partition(update_partition);
				if (err != ESP_OK) {
					ESP_LOGD(LOG_TAG, "esp_ota_set_boot_partition failed (%s)!", esp_err_to_name(err));
				}
			}
			break;
		default:
			break;
		}

		// return status
		eStatus = g_eUpdateResponse;
		// get package id
		*pucCurrentPackageID = g_ucCurrentPackageID;
		g_eUpdateResponse = Response_Null;
	}

	return eStatus;
}

static void Update_GWTimeoutHandler(TimerHandle_t xTimer)
{
	g_bGWUpdate = false;
	esp_ota_end(update_handle);
	gBCurrent = 0;
}
