/* Includes ------------------------------------------------------------------*/
#include "sensor_update.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/timers.h"
#include "acc_pin.h"
#include "wit_config.h"

#define SENSOR_UPDATE_MIN_SIZE 10
#define SENSOR_UPDATE_TIMEOUT 60000

static bool g_bSensorUpdate = false;

static bool g_bSensorFinished = false;

static TimerHandle_t g_xSensorUpdateTimer = NULL;

static void Sensor_TimeoutHandler(TimerHandle_t xTimer);

eMBErrorCode Sensor_UpdateHandler(unsigned char ucSensorAddr, unsigned char ucSensorID, unsigned char *pucData, unsigned long ulSize,
								  unsigned char *pucResponse, unsigned char *pucCnt)
{
	// Command(1 Byte) | Address(2 Byte) | Count(2 Byte) | Data(Count * 2 Byte) | Package Id(2 Bytes) | CheckSum(1 Byte)
	unsigned char ucCheckSum = 0;
	int i;
	eMBErrorCode ret = MB_EINVAL;
	unsigned int retryCnt = 0;

	if (ulSize < SENSOR_UPDATE_MIN_SIZE)
	{
		return MB_EINVAL;
	}

	for (i = 0; i < ulSize - 1; i++)
	{
		ucCheckSum += pucData[i];
	}

	if (ucCheckSum != pucData[ulSize - 1])
		return MB_EINVAL;

	unsigned char ucRecvCommand;
	unsigned short *pusRecvData, usRecvAddress, usRecvCount;

	// Set flag and reset timer
	g_bSensorUpdate = true;
	if (g_xSensorUpdateTimer == NULL)
	{
		g_xSensorUpdateTimer = xTimerCreate("SensorTimer", SENSOR_UPDATE_TIMEOUT, pdFALSE, NULL, Sensor_TimeoutHandler);
	}
	if (g_xSensorUpdateTimer != NULL)
		xTimerStart(g_xSensorUpdateTimer, 1000);

	ucRecvCommand = pucData[0];
	usRecvAddress = (unsigned short)pucData[1] << 8 | pucData[2];

	usRecvCount = (unsigned short)pucData[3] << 8 | pucData[4];
	*pucCnt = 0;
	if (ucRecvCommand == MB_FUNC_WRITE_MULTIPLE_REGISTERS ||
		ucRecvCommand == MB_FUNC_WRITE_REGISTER)
	{
		if (usRecvCount * 2 + SENSOR_UPDATE_MIN_SIZE - 2 != ulSize)
			return MB_EINVAL;
	}
	else if (ulSize != SENSOR_UPDATE_MIN_SIZE)
	{
		return MB_EINVAL;
	}
	else
	{
		*pucCnt = usRecvCount;
	}

	pusRecvData = (unsigned short *)&pucData[5];
	unsigned short pkgId = ((unsigned short)pucData[ulSize - 3] << 8) | (unsigned short)pucData[ulSize - 2];

	if (usRecvAddress == 0x5000)
	{
		retryCnt = 0;
		while (retryCnt < 5)
		{
			ret = eMBWriteHoldingRegister(ACC_ADDR_TO_PORT(ucSensorAddr), ucSensorID, 0x5078, pkgId);
			if (ret == MB_EMASTERNOERR)
			{
				// Set package id failed
				break;
			}
			retryCnt++;
		}
	}

	retryCnt = 0;
	while (retryCnt < 5)
	{
		// Update sensor
		switch (ucRecvCommand)
		{
		case MB_FUNC_READ_HOLDING_REGISTER:
			ret = eMBReadHoldingRegister(ACC_ADDR_TO_PORT(ucSensorAddr), ucSensorID, usRecvAddress, usRecvCount, (unsigned short *)pucResponse);
			break;
		case MB_FUNC_WRITE_REGISTER:
			ret = eMBWriteHoldingRegister(ACC_ADDR_TO_PORT(ucSensorAddr), ucSensorID, usRecvAddress, pusRecvData[0]);
			break;
		case MB_FUNC_WRITE_MULTIPLE_REGISTERS:
			ret = eMBWriteMultiHoldingRegister(ACC_ADDR_TO_PORT(ucSensorAddr), ucSensorID, usRecvAddress, usRecvCount, pusRecvData);
			break;
		default:
			ret = MB_EINVAL;
			break;
		}

		if (ret == MB_EMASTERNOERR)
		{
			break;
		}
		retryCnt++;
	}

	g_bSensorFinished = (usRecvAddress == 0x5079) ? true : false;

	return ret;
}

bool Sensor_GetUpdateStatus()
{
	return g_bSensorUpdate;
}

bool Sensor_GetFinishedStatus()
{
	bool bStatus;

	bStatus = g_bSensorFinished;
	if (g_bSensorFinished == true)
	{
		g_bSensorFinished = false;
	}
	return bStatus;
}

static void Sensor_TimeoutHandler(TimerHandle_t xTimer)
{
	g_bSensorUpdate = false;
}
