#include "fsad.h"
#include <string.h>
#include <stdint.h>
#include <stdio.h>
#include "esp_log.h"

static char *data_buf   = NULL;
static char *access_buf = NULL;

static unsigned short fsad_apkg_checksum(char const *ucData, unsigned short usSize);
/**
 *                         生成访问帧
 * @param    ucCode        访问帧的功能码
 * @param    usSzie        访问的数据大小
 * @return                 char 返回访问帧数据的地址（使用后使用free） 
 **/
char  *fsad_get_request_pack(unsigned int usCode, unsigned int usSize)
{
    access_frame frame;
    memcpy(frame.head, "WITR", 4);
    frame.code        = usCode;
    frame.data_size   = usSize;
    memcpy(frame.tail, "\n\r", 2);
    if(access_buf != NULL){
        free(access_buf);
    }
    access_buf = (char*) malloc(sizeof(access_frame));
    memcpy(access_buf, &frame, sizeof(access_frame));
    // esp_log_buffer_hex("cm", access_buf, sizeof(access_frame) - 2);  
    return access_buf; 
}

/**
 *                                生成数据帧
 * @param    ucCode               数据帧的功能码
 * @param    current_id           当前包id
 * @param    total_pack_id        总包个数
 * @param    data_size            发送的净数据的sizeof
 * @param    data                 发送的净数据
 * @param    size                 数据帧的size
 * @return                        data_frame 返回数据帧数据的地址（使用后使用free） 
 **/
char *fsad_get_data_pack(unsigned int usCode, unsigned int current_id, 
                        unsigned int total_pack_id, 
                        unsigned short data_size, char *data, unsigned int *size)
{
    unsigned int uiSize = 0;
    data_frame frame;
    unsigned int ui_pack_size = 0;
    uiSize = sizeof(frame.head) + sizeof(frame.code) + sizeof(frame.current_id) + sizeof(frame.total_pack_id) + sizeof(frame.data_size);        
    *size = uiSize  + data_size  + sizeof(frame.crc) + sizeof(frame.tail);
    if(data_buf != NULL) {
        free(data_buf);
    }
    data_buf = (char*) malloc(*size);
    memcpy(frame.head, "WITW",  sizeof(frame.head));
    frame.code          = usCode;
    frame.current_id    = current_id;
    frame.total_pack_id = total_pack_id;
    frame.data_size     = data_size;
    frame.data          = data;
    frame.crc           = fsad_apkg_checksum(frame.data, frame.data_size);
    memcpy(frame.tail, "\n\r", 2);
    memcpy(data_buf, &frame, uiSize);
    memcpy(data_buf + uiSize, frame.data, frame.data_size);
    ui_pack_size += uiSize + frame.data_size;
    memcpy(data_buf + ui_pack_size, &frame.crc, sizeof(frame.crc));
    ui_pack_size += sizeof(frame.crc);
    memcpy(data_buf + ui_pack_size, &frame.tail, sizeof(frame.tail));
    return data_buf;
}

/**
 *                                 数据帧解析
 * @param    sorData               源数据地址
 * @param    sorSize               源数据大小
 * @param    dafr                  目的数据地址
 * @return                         eWCPinfo 返回数据帧信息
 **/
eWCPinfo fsad_data_frame_parsing(void const *sorData, unsigned short sorSize, data_frame *dafr)
{
    unsigned short usCnt = 0;
	memcpy(dafr->head, (char *)sorData, sizeof(dafr->head));
    usCnt += sizeof(dafr->head);
    memcpy(&(dafr->code), ((char *)sorData) + usCnt, sizeof(dafr->code));
    usCnt += sizeof(dafr->code);
    memcpy(&(dafr->current_id), ((char *)sorData) + usCnt, sizeof(dafr->current_id));
    usCnt += sizeof(dafr->current_id);
    memcpy(&(dafr->total_pack_id), ((char *)sorData) + usCnt, sizeof(dafr->total_pack_id));
    usCnt += sizeof(dafr->total_pack_id);
    memcpy(&(dafr->data_size), ((char *)sorData) + usCnt, sizeof(dafr->data_size));
    usCnt += sizeof(dafr->data_size);
    dafr->data = ((char *)sorData) + usCnt;
    memcpy(&(dafr->crc), ((char *)sorData) + usCnt + dafr->data_size, sizeof(dafr->crc));
    usCnt += sizeof(dafr->crc); 
    memcpy(&(dafr->tail), ((char *)sorData) + usCnt + dafr->data_size, sizeof(dafr->tail));
	if(dafr->tail[0] != 0x0A && dafr->tail[1] != 0x0D) {
		return WCP_FRAME_NOT_COMPLETE;
	}
	if(fsad_apkg_checksum(dafr->data, dafr->data_size) != dafr->crc) {
		return WCP_FRAME_CRC_ERROR;
	}
	return WCP_OK;
}

/**
 *                                 访问帧解析
 * @param    sorData               源数据地址
 * @param    sorSize               源数据大小
 * @param    acfr                  目的数据地址
 * @return                         eWCPinfo 返回数据帧信息
 **/
eWCPinfo fsad_access_frame_parsing(void const *sorData, unsigned short sorSize, access_frame *acfr)
{
    access_frame *af = NULL;
    af = (access_frame *)sorData;
    if(af->tail[0] != 0x0A && af->tail[1] != 0x0D) {
        acfr = NULL;
		return WCP_FRAME_NOT_COMPLETE;
	}
    sprintf(acfr->head, af->head);
    acfr->code = af->code;
    acfr->data_size = af->data_size;
    sprintf(acfr->tail, af->tail);
    return WCP_OK;
}

/**
 *                         确认帧类型
 * @param    ucData        一包数据
 * @return                 eWCPoperate 返回帧类型 ADDR_READ：访问帧；ADDR_WRITE：数据帧；ADDR_ERROR：未知帧
 **/
eWCPoperate fsad_confirm_the_frame_type(char const *ucData)
{
    if(strstr(ucData, "WITR") != NULL)
    {
        return ADDR_READ;
    }
    else if(strstr(ucData, "WITW") != NULL)
    {
        return ADDR_WRITE;  
    }
    return ADDR_ERROR;
}

/**
 *                          数据帧净载数据校验
 * @param    ucData         净载数据
 * @param    usSize         净载数据的sizeof
 * @return                 返回校验的值CheckSum
 **/
static unsigned short fsad_apkg_checksum(char const *ucData, unsigned short usSize)
{
    unsigned short sum = 0; 
    for(int i = 0 ; i < usSize ; i++ ) {
        sum += ucData[i];
    }
    return sum;
}


