#include "flash.h"
#include "esp_vfs_fat.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"

#include "esp_partition.h"
#include "esp_log.h"

#define RTOS_CREATE_SIGNAL(signal)		signal = xSemaphoreCreateBinary()
#define RTOS_TAKE(signal)				xSemaphoreTake(signal, portMAX_DELAY)
#define RTOS_GIVE(signal)				xSemaphoreGive(signal)

#define FLASH_GW_CONFIG_MAX_SIZE      (52 * 1024)
#define FLASH_GW_UP_CONFIG_MAX_SIZE   (52 * 1024)
#define FLASH_SEN_CONFIG_MAX_SIZE     (100 * 1024)
#define FLASH_SEN_BP_CONFIG_MAX_SIZE  (100 * 1024)
#define FLASH_PVD_CONFIG_MAX_SIZE     (4 * 1024)
#define FLASH_THRESHOLD_MAX_SIZE      (200 * 1024)
#define FLASH_THRESHOLD_BP_MAX_SIZE   (200 * 1024)
#define FLASH_THRESHOLD_FLAG_MAX_SIZE (4 * 1024)
#define FLASH_FEAHISTORY_MAX_SIZE     (200 * 1024)
#define FLASH_GW_FEATURE_MAX_SIZE     (200 * 1024)
#define FLASH_SENSOR_FEA1_MAX_SIZE    (200 * 1024)
#define FLASH_SENSOR_FEA2_MAX_SIZE    (200 * 1024)
#define FLASH_SENSOR_FEA3_MAX_SIZE    (200 * 1024)
#define FLASH_SENSOR_FEA4_MAX_SIZE    (200 * 1024)
#define FLASH_SENSOR_FEA5_MAX_SIZE    (200 * 1024)
#define FLASH_SENSOR_FEA6_MAX_SIZE    (200 * 1024)
#define FLASH_BLEBIN_MAX_SIZE         (600 * 1024)


static SemaphoreHandle_t           gFlashSemaphore;
static bool gFlashStatus                  = true;									   
static const esp_partition_t *blebin      = NULL;
static const esp_partition_t *gconfig     = NULL;
static const esp_partition_t *gconfigBP   = NULL;
static const esp_partition_t *sconfig     = NULL;
static const esp_partition_t *sconfigBP   = NULL;
static const esp_partition_t *pvd         = NULL;

void flash_semphr_init(void)
{ 
    RTOS_CREATE_SIGNAL(gFlashSemaphore);
	RTOS_GIVE(gFlashSemaphore);
  
    gconfig   = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "gconfig");
    gconfigBP = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "gconfigBP");
    sconfig   = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "sconfig");
    sconfigBP = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "sconfigBP");
    pvd       = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "pvd");
    blebin    = esp_partition_find_first(ESP_PARTITION_TYPE_DATA, ESP_PARTITION_SUBTYPE_ANY, "blebin");
    if (blebin == NULL)
    {
        ESP_LOGD("flash","flash init fail");
        gFlashStatus = false;
    }     
}

void write_flash(EFLASH_TYPE eft, const void *data, const size_t size)
{
    RTOS_TAKE(gFlashSemaphore);
    esp_err_t ret;
    if(gFlashStatus == true)
    {
        switch(eft)
        {
            case E_GW_CONFIG:
                ret = esp_partition_erase_range(gconfig, 0, FLASH_GW_CONFIG_MAX_SIZE);
                if (ret != ESP_OK) {
                }
                ret = esp_partition_write(gconfig, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("gconfig","Partition write failed");
                }
                break;
            case E_GWBP_CONFIG:
                ret = esp_partition_erase_range(gconfigBP, 0, FLASH_GW_UP_CONFIG_MAX_SIZE);
                if (ret != ESP_OK) {
                    ESP_LOGD("gconfigBP","erase failed");
                }
                ret = esp_partition_write(gconfigBP, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("gconfigBP","Partition write failed");
                }
                break;
            case E_SEN_CONFIG:
                ret = esp_partition_erase_range(sconfig, 0, FLASH_SEN_CONFIG_MAX_SIZE);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfig","erase failed");
                }
                ret = esp_partition_write(sconfig, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfig","Partition write failed");
                }
                break;
            case E_SENBP_CONFIG:
                ret = esp_partition_erase_range(sconfigBP, 0, FLASH_SEN_BP_CONFIG_MAX_SIZE);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfigBP","erase failed");
                }
                ret = esp_partition_write(sconfigBP, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfigBP","Partition write failed");
                }
                break;
            case E_PVD_STORE:
                ret = esp_partition_erase_range(pvd, 0, FLASH_PVD_CONFIG_MAX_SIZE);
                if (ret != ESP_OK) {
                    ESP_LOGD("pconfig","erase failed");
                }
                ret = esp_partition_write(pvd, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("pconfig","Partition write failed");
                }
                break;
            default:
                break;
        }
    }
    RTOS_GIVE(gFlashSemaphore);
}

void read_flash(EFLASH_TYPE eft, void *data, const size_t size)
{
    RTOS_TAKE(gFlashSemaphore);
    esp_err_t ret;
    if(gFlashStatus == true)
    {
        switch(eft)
        {
            case E_GW_CONFIG:
                ret = esp_partition_read(gconfig, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("gconfig","Partition read failed");
                }
                break;
            case E_GWBP_CONFIG:
                ret = esp_partition_read(gconfigBP, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("gconfigBP","Partition read failed");
                }
                break;
            case E_SEN_CONFIG:
                ret = esp_partition_read(sconfig, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfig","Partition read failed");
                }
                break;
            case E_SENBP_CONFIG:
                ret = esp_partition_read(sconfigBP, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("sconfigBP","Partition read failed");
                }
                break;
            case E_PVD_STORE:
                ret = esp_partition_read(pvd, 0, data, size);
                if (ret != ESP_OK) {
                    ESP_LOGD("pvd","Partition read failed");
                }
                break;
            default:
                break;
        }
    }
    RTOS_GIVE(gFlashSemaphore);
}

 esp_err_t read_ble_bin(char *data, size_t offset, size_t size)
{
    RTOS_TAKE(gFlashSemaphore);
    esp_err_t ret;
    ret = esp_partition_read(blebin, offset, data, size);
    if (ret != ESP_OK) {
        ESP_LOGD("ble_bin","Partition read failed");
    }
    RTOS_GIVE(gFlashSemaphore);
    return ret;
}

 esp_err_t write_ble_bin(char *data, size_t offset, size_t size)
{
    RTOS_TAKE(gFlashSemaphore);
    esp_err_t ret;
    ret = esp_partition_write(blebin, offset, data, size);
    if (ret != ESP_OK) {
        ESP_LOGD("ble_bin","Partition write failed");
    }
    RTOS_GIVE(gFlashSemaphore);
    return ret;
}

void erase_bin(void)
{
    RTOS_TAKE(gFlashSemaphore);
    esp_err_t ret;
    ret = esp_partition_erase_range(blebin, 0, 614400);
	if (ret != ESP_OK) {
		ESP_LOGD("ble_bin","erase failed");
	}
    RTOS_GIVE(gFlashSemaphore);
}

bool getFalshStatus(void)
{
    return gFlashStatus;
}

void setFlashStatus(bool bStatus)
{
    gFlashStatus = bStatus;
}


