#include <stdio.h>
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "esp_system.h"
#include "common.h"
#include "rtc.h"
#include "esp_log.h"
#include "esp_sntp.h"

#define PCF8563_I2C_ADDR    0x51
#define PCF8563_BAK_ADDR    0x08

#define RTC_RTOS_CREATE_SIGNAL(signal)		signal = xSemaphoreCreateBinary()
#define RTC_RTOS_TAKE(signal)				xSemaphoreTake(signal, portMAX_DELAY)
#define RTC_RTOS_GIVE(signal)				xSemaphoreGive(signal)

static SemaphoreHandle_t           gRTCSemaphore;

void pcf8563_init(i2c_port_t i2c_port)
{
    i2c_config_t conf = {
        .mode = I2C_MODE_MASTER,
        .sda_io_num = GPIO_NUM_16,
        .sda_pullup_en = GPIO_PULLUP_ENABLE,
        .scl_io_num = GPIO_NUM_15,
        .scl_pullup_en = GPIO_PULLUP_ENABLE,
        .master.clk_speed = 100000
    };
    RTC_RTOS_CREATE_SIGNAL(gRTCSemaphore);
	RTC_RTOS_GIVE(gRTCSemaphore);

    i2c_param_config(i2c_port, &conf);
    i2c_driver_install(i2c_port, conf.mode, 0, 0, 0);

    uint8_t buf[2];
    buf[0] = 0x00;  // control/status1 register
    buf[1] = 0x00;  // disable alarm interrupts
    i2c_cmd_handle_t cmd = i2c_cmd_link_create();
    i2c_master_start(cmd);
    i2c_master_write_byte(cmd, PCF8563_I2C_ADDR << 1 | I2C_MASTER_WRITE, true);
    i2c_master_write(cmd, buf, 2, true);
    i2c_master_stop(cmd);
    i2c_master_cmd_begin(i2c_port, cmd, 1000 / portMAX_DELAY);
    i2c_cmd_link_delete(cmd);
}

void pcf8563_get_time(i2c_port_t i2c_port, struct tm *timeinfo)
{
    RTC_RTOS_TAKE(gRTCSemaphore);
    uint8_t buf[7];
    buf[0] = 0x02;  // start at seconds register
    i2c_cmd_handle_t cmd = i2c_cmd_link_create();
    i2c_master_start(cmd);
    i2c_master_write_byte(cmd, PCF8563_I2C_ADDR << 1 | I2C_MASTER_WRITE, true);
    i2c_master_write(cmd, &buf[0], 1, true);
    i2c_master_start(cmd);
    i2c_master_write_byte(cmd, PCF8563_I2C_ADDR << 1 | I2C_MASTER_READ, true);
    i2c_master_read(cmd, &buf[0], 7, I2C_MASTER_LAST_NACK);
    i2c_master_stop(cmd);
    i2c_master_cmd_begin(i2c_port, cmd, 1000 / portMAX_DELAY);
    i2c_cmd_link_delete(cmd);

    timeinfo->tm_sec = ((buf[0] & 0x70) >> 4) * 10 + (buf[0] & 0x0F);
    timeinfo->tm_min = ((buf[1] & 0x70) >> 4) * 10 + (buf[1] & 0x0F);
    timeinfo->tm_hour = ((buf[2] & 0x30) >> 4) * 10 + (buf[2] & 0x0F);
    timeinfo->tm_mday = ((buf[3] & 0x30) >> 4) * 10 + (buf[3] & 0x0F);
    timeinfo->tm_wday = buf[4] & 0x07;
    timeinfo->tm_mon  = (buf[5] & 0x1F);
    timeinfo->tm_year = ((buf[6] & 0xF0) >> 4) * 10 + (buf[6] & 0x0F);
    RTC_RTOS_GIVE(gRTCSemaphore);
}

void pcf8563_set_time(i2c_port_t i2c_port, struct tm *timeinfo)
{
    RTC_RTOS_TAKE(gRTCSemaphore);
    uint8_t buf[8];
    buf[0] = 0x02; // start at seconds register
    buf[1] = (timeinfo->tm_sec / 10 << 4) | (timeinfo->tm_sec % 10);
    buf[2] = (timeinfo->tm_min / 10 << 4) | (timeinfo->tm_min % 10);
    buf[3] = (timeinfo->tm_hour / 10 << 4) | (timeinfo->tm_hour % 10);
    buf[4] = (timeinfo->tm_mday / 10 << 4) | (timeinfo->tm_mday % 10);
    buf[5] = (timeinfo->tm_wday & 0x07);
    buf[6] = timeinfo->tm_mon;
    buf[7] = (timeinfo->tm_year / 10 << 4) | (timeinfo->tm_year % 10);
    i2c_cmd_handle_t cmd = i2c_cmd_link_create();
    i2c_master_start(cmd);
    i2c_master_write_byte(cmd, PCF8563_I2C_ADDR << 1 | I2C_MASTER_WRITE, true);
    i2c_master_write(cmd, buf, 8, true);
    i2c_master_stop(cmd);
    i2c_master_cmd_begin(i2c_port, cmd, 1000 / portMAX_DELAY);
    i2c_cmd_link_delete(cmd);
    RTC_RTOS_GIVE(gRTCSemaphore);
}

void RTC_InternalSetTimeStamp(unsigned long timestamp)
{
    struct tm timeinfo;
    Calendar Calendar;
    // time_t t;
    struct timeval now;
    TimeStampToTime(timestamp, &Calendar);
    timeinfo.tm_year = Calendar.usYear - 2000;
    timeinfo.tm_mon = Calendar.ucMonth;
    timeinfo.tm_mday = Calendar.ucDay;
    timeinfo.tm_hour = Calendar.ucHour;
    timeinfo.tm_min = Calendar.ucMinute;
    timeinfo.tm_sec = Calendar.ucSecond;
    pcf8563_set_time(I2C_NUM_0, &timeinfo);
    now.tv_sec = timestamp;
    now.tv_usec = 0;
    settimeofday(&now, NULL);
}

unsigned long RTC_InternalGetTimeStamp(void)
{
    struct tm timeinfo;
    static unsigned long old_time = 0;
    time_t now;
    unsigned long ulTime = 0;
    pcf8563_get_time(I2C_NUM_0, &timeinfo);
    Calendar Calendar;  
    Calendar.usYear   = timeinfo.tm_year + 2000;
    Calendar.ucMonth  = timeinfo.tm_mon;
    Calendar.ucDay    = timeinfo.tm_mday;
    Calendar.ucHour   = timeinfo.tm_hour;
    Calendar.ucMinute = timeinfo.tm_min;
    Calendar.ucSecond = timeinfo.tm_sec; 
    ulTime = TimeToTimeStamp(&Calendar);
    if(old_time >= ulTime || ulTime <= 1640966400)
    {
        old_time = ulTime;
        time(&now);
        ulTime = now;
    }
    else 
    {  
        old_time = ulTime;
    }
    return ulTime;
}


