#include "spi.h"
#include "esp_log.h"
#include <string.h>
#include "esp_rom_gpio.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
/* Private macros ------------------------------------------------------------*/

/* Private types -------------------------------------------------------------*/
#define W5100S_HOST     SPI2_HOST
#define W5100S_NUM_MISO 13
#define W5100S_NUM_MOSI 11
#define W5100S_NUM_CLK  12
#define W5100S_NUM_CS   10

#define RTOS_CREATE_SIGNAL(signal)		signal = xSemaphoreCreateBinary()
#define RTOS_TAKE(signal)				xSemaphoreTake(signal, portMAX_DELAY)
#define RTOS_GIVE(signal)				xSemaphoreGive(signal)

static SemaphoreHandle_t       gSPISemaphore;

static void spi_gpio_init(void);
static esp_err_t spi_write(uint32_t addrSel, void *wb, uint32_t len);
static esp_err_t spi_read(uint32_t addrSel, void* out_data, uint32_t len);

static spi_device_handle_t spi;

void spi_init(unsigned long ulPrescaler)
{
    // esp_err_t ret;
    spi_device_interface_config_t devcfg={
        .address_bits = 16, 
        .command_bits = 8, 
        .spics_io_num = -1,
        .mode = 0,
        .clock_speed_hz = ulPrescaler,
        .queue_size = 20, 
    };
    spi_gpio_init();
    spi_bus_add_device(W5100S_HOST,&devcfg, &spi);
    RTOS_CREATE_SIGNAL(gSPISemaphore);
	RTOS_GIVE(gSPISemaphore);
}

static esp_err_t spi_write(uint32_t addrSel, void *wb, uint32_t len)
{
    RTOS_TAKE(gSPISemaphore);
    esp_err_t err;
    err = spi_device_acquire_bus(spi, portMAX_DELAY);
    if (err != ESP_OK) return err;
    spi_transaction_t t;
    memset(&t, 0, sizeof(t));       //Zero out the transaction 
    t.cmd = 0xF0;
    t.addr = addrSel;
    t.length = 8 * len;
    t.tx_buffer = wb;     
    t.user = (void*)1;
    err = spi_device_polling_transmit(spi, &t);
   
    if (err != ESP_OK) {
        ESP_LOGD("spi", "esp_err_t %d...", err);
    }
    // esp_log_buffer_hex("spi write",t.tx_buffer, len);
    spi_device_release_bus(spi);
    RTOS_GIVE(gSPISemaphore);
    return err;
}

static esp_err_t spi_read(uint32_t addrSel, void* out_data, uint32_t len)
{
    RTOS_TAKE(gSPISemaphore);
    uint8_t *in_data = NULL;
    spi_transaction_t t;
    memset(&t, 0, sizeof(t));       //Zero out the transaction 
    in_data = (uint8_t*)malloc(len*sizeof(uint8_t));// 使用malloc分配内存的首地址
    memset(in_data, 0xff, len); 
    t.cmd  = 0x0F;
    t.addr = addrSel;
    t.length = 8 * len;
    t.tx_buffer = in_data;
    t.rx_buffer = out_data;
    t.user = (void*)1;
    esp_err_t err = spi_device_polling_transmit(spi, &t);
    // esp_log_buffer_hex("spi read",t.rx_buffer, len);
    free(in_data);    //释放掉使用的内存地址
    in_data = NULL;
    RTOS_GIVE(gSPISemaphore);
    if (err!= ESP_OK) return err;
    return ESP_OK;
}

static void spi_gpio_init(void)
{
    esp_err_t ret;
    spi_bus_config_t buscfg = {
        .miso_io_num = W5100S_NUM_MISO,
        .mosi_io_num = W5100S_NUM_MOSI,
        .sclk_io_num = W5100S_NUM_CLK,
        .quadwp_io_num = -1,
        .quadhd_io_num = -1,
        .max_transfer_sz = 2048,                    // 最大传输数据大小
    };
    //Initialize the SPI bus
    ret = spi_bus_initialize(W5100S_HOST, &buscfg, SPI_DMA_CH_AUTO);
    ESP_ERROR_CHECK(ret);
    esp_rom_gpio_pad_select_gpio(W5100S_NUM_CS);   // 选择cs个GPIO
	gpio_set_direction(W5100S_NUM_CS, GPIO_MODE_OUTPUT); // 把这个GPIO作为输出  
}

void spi_send_byte(uint32_t addrSel, uint8_t *wb, uint32_t len)
{
    spi_write(addrSel, wb, len);  
}

void spi_read_byte(uint32_t addrSel, uint8_t * out_data, uint32_t len)
{ 
    spi_read(addrSel, out_data, len);  
}


void cs_low(void)
{
  gpio_set_level(W5100S_NUM_CS, 0);
}

void cs_high(void)
{
  gpio_set_level(W5100S_NUM_CS, 1);
}
