/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "string.h"
#include "usart.h"
#include "esp_log.h"

/* Private macros ------------------------------------------------------------*/
#define UART_MAX_PORT_NUM				2

/* Private types -------------------------------------------------------------*/
typedef struct
{
	uart_port_t uartx;
	void (*UARTx_ConfigureGPIO)();
} UART;

typedef struct
{
	void (*pFuncRxneHandler)(void);
	void (*pFuncIdleHandler)(void);
	void (*pFuncTxeHandler)(void);
	void (*pFuncTcHandler)(void);
} UART_Handler;

/* Private functions ---------------------------------------------------------*/
static void UART1_ConfigureGPIO(void);
static void UART2_ConfigureGPIO(void);
static void uart_event_task(void *pvParameters);
/* Private constants ---------------------------------------------------------*/
static const UART g_xUART[UART_MAX_PORT_NUM + 1] = {
	[0] = {
		.uartx = 0,
		.UARTx_ConfigureGPIO = NULL,
	},
	[1] = {
		.uartx = UART_NUM_0,
		.UARTx_ConfigureGPIO = UART1_ConfigureGPIO,
	},
	[2] = {
		.uartx = UART_NUM_1,
		.UARTx_ConfigureGPIO = UART2_ConfigureGPIO,
	}
};

/* Exported user handlers ----------------------------------------------------*/
static UART_Handler g_xUARTHandler[UART_MAX_PORT_NUM + 1];

static bool g_bIdle = false;
static bool g_bTxe  = false;
static bool g_bTc   = false;
static bool g_bRxne = false;
/**
  * @brief  UART Init
  * @param  ulPort: 1, 2, 3, 4, 5
  * @param  ulBaudRate: UARTx Baudrate
  * @param  ulParity: UARTx Parity
  * @retval None
  */
void UARTx_Init(unsigned long ulPort, unsigned long ulBaudRate, unsigned long ulParity, unsigned long ulStopBits)
{
	if (ulPort > UART_MAX_PORT_NUM)
		return;
	
	const uart_config_t uart_config = {
        .baud_rate = ulBaudRate,
        .data_bits = UART_DATA_8_BITS,
        .parity = ulParity,
        .stop_bits = ulStopBits,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE,
        .source_clk = UART_SCLK_APB,
    };

    // We won't use a buffer for sending data.
    uart_driver_install(g_xUART[ulPort].uartx, 1024, 0, 0, NULL, 0);
    uart_param_config(g_xUART[ulPort].uartx, &uart_config);
    g_xUART[ulPort].UARTx_ConfigureGPIO();
    xTaskCreate(uart_event_task, "uart_event_task", 2048, NULL, tskIDLE_PRIORITY + 3, NULL);
    ESP_LOGI("uart 78","%ld", ulPort);
}


static void uart_event_task(void *pvParameters)
{
  while(1)
  {
    if(g_bIdle == true)
    {
      if(g_xUARTHandler[2].pFuncIdleHandler != NULL)
		    g_xUARTHandler[2].pFuncIdleHandler();
    }
    if(g_bTxe == true)
    {
      if(g_xUARTHandler[2].pFuncTxeHandler != NULL)
		    g_xUARTHandler[2].pFuncTxeHandler();
    }
    if(g_bTc == true)
    {
      if(g_xUARTHandler[2].pFuncTcHandler != NULL)
		    g_xUARTHandler[2].pFuncTcHandler();
    }
    if(g_bRxne == true)
    {
      if(g_xUARTHandler[2].pFuncRxneHandler != NULL)
		    g_xUARTHandler[2].pFuncRxneHandler();
    }
    vTaskDelay(1 / portTICK_PERIOD_MS);
  }
}
/**
  * @brief  UART IT ConfigureUARTx_EnableIT
  * @param  ulPort: 1, 2, 3, 4, 5
  * @param  bIdle: IT_IDLE
  * @param  bTxe: IT_TXE
  * @param  bTc: IT_TC
  * @param  bRxne: IT_RXNE
  * @retval None
  */
void UARTx_EnableIT(unsigned long ulPort, bool bIdle, bool bTxe, bool bTc, bool bRxne)
{
	if (ulPort > UART_MAX_PORT_NUM)
		return;
  // bIdle == true ? uart_enable_intr_mask(ulPort, UART_INTR_TX_BRK_IDLE) : uart_disable_intr_mask(ulPort, UART_INTR_TX_BRK_IDLE);  
  // bTxe  == true ? uart_enable_intr_mask(ulPort, UART_INTR_TXFIFO_EMPTY) : uart_disable_intr_mask(ulPort, UART_INTR_TXFIFO_EMPTY);  
  // bTc   == true ? uart_enable_intr_mask(ulPort, UART_INTR_TX_DONE) : uart_disable_intr_mask(ulPort, UART_INTR_TX_DONE);  
  // bRxne == true ? uart_enable_intr_mask(ulPort, UART_INTR_RXFIFO_FULL | UART_INTR_RXFIFO_TOUT) : uart_disable_intr_mask(ulPort, UART_INTR_RXFIFO_FULL | UART_INTR_RXFIFO_TOUT); 
  g_bIdle = bIdle;
  g_bTxe  = bTxe;
  g_bTc   = bTc;
  g_bRxne = bRxne;
}

/**
  * @brief  UART IT Configure
  * @param  ulPriority: UART IT Priority
  * @retval None
  */
void UARTx_SetPriority(unsigned long ulPort, unsigned long ulPriority)
{
  // esp32S3不支持中断类型及优先级的配置
	if (ulPort > UART_MAX_PORT_NUM)
		return;
}

/**
  * @brief  UART Send Data (Blocked)
  * @param  ulPort: 1, 2, 3, 4, 5
  * @param  pucData: Data pointer
  * @param  ulLen: Length of data
  * @retval None
  */
void UARTx_Send(unsigned long ulPort, unsigned char * pucData, unsigned long ulLen)
{
	uart_write_bytes(g_xUART[ulPort].uartx, pucData, ulLen);
  esp_log_buffer_hex("uart tx", pucData, ulLen);
}


/**
  * @brief  UART Send Data (None Blocked for IT)
  * @param  ulPort: 1, 2, 3, 4, 5
  * @param  ucData: Data
  * @retval None
  */
void UARTx_SendToDR(unsigned long ulPort, unsigned char ucData)
{
	if (ulPort > UART_MAX_PORT_NUM)
		return;	
	uart_write_bytes(g_xUART[ulPort].uartx, &ucData, 1);
  esp_log_buffer_hex("uart tx", &ucData, 1);
}

/**
  * @brief  UART Receive Data
  * @param  ulPort: 1, 2, 3, 4, 5
  * @retval received data
  */
unsigned char UARTx_Receive(unsigned long ulPort)
{
  uint8_t buf = 0;
	if (ulPort > UART_MAX_PORT_NUM)
		return 0;
	uart_read_bytes(g_xUART[ulPort].uartx, &buf, 1, portTICK_PERIOD_MS);
	return buf;
}

/**
  * @brief  UART set call back function
  * @param  ulPort: 1, 2, 3, 4, 5
  * @retval received data
  */
void UARTx_SetCallBack(unsigned long ulPort, void (*pFuncIdleCallBack)(void), void (*pFuncTxeCallBack)(void), void (*pFuncTcCallBack)(void), void (*pFuncRxneCallBack)(void))
{
	if (ulPort > UART_MAX_PORT_NUM)
		return;

	g_xUARTHandler[ulPort].pFuncIdleHandler = pFuncIdleCallBack;
	g_xUARTHandler[ulPort].pFuncTxeHandler	= pFuncTxeCallBack;
	g_xUARTHandler[ulPort].pFuncTcHandler	= pFuncTcCallBack;
	g_xUARTHandler[ulPort].pFuncRxneHandler	= pFuncRxneCallBack;
}
/**
  * @brief  UART1 GPIO Configure
  * @param  None
  * @retval None
  */
static void UART1_ConfigureGPIO(void)
{
	uart_set_pin(UART_NUM_0, GPIO_NUM_43, GPIO_NUM_44, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
}
/**
  * @brief  UART2 GPIO Configure
  * @param  None
  * @retval None
  */
static void UART2_ConfigureGPIO(void)
{
  uart_set_pin(UART_NUM_1, GPIO_NUM_17, GPIO_NUM_18, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);
}

