/* Includes ------------------------------------------------------------------*/
#include "mqtt_port.h"
/**
  * @brief  创建MQTT Socket
  * @param  pcMode: "TCP" "UDP"等
  * @param  pcHost: IP地址或域名
  * @param  pcPort: 端口号
  * @retval 返回Socket句柄
  */
long MQTT_CreateSocket(char *pcMode, char *pcHost, char *pcPort)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	return ETHERNET_CreateSocket(pcMode, pcHost, pcPort);
#elif CONFIG_EC200S_USB_4G > 0
	return Celluar_CreateSocket(pcMode, pcHost, pcPort);
#else
  return 0;
#endif
}

/**
  * @brief  释放MQTT Socket
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval None
  */
void MQTT_ReleaseSocket(unsigned long ulSocketID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	ETHERNET_ReleaseSocket(ulSocketID);
#elif CONFIG_EC200S_USB_4G > 0
	Celluar_ReleaseSocket(ulSocketID);
#endif
}

/**
  * @brief  把重连函数赋值给底层，供底层连接上MQTT后直接调用对应接口重连
  * @param  pFunction: 重连函数
  * @retval None
  */
void MQTT_ReconnectFuncAssignFor(bool (*pFunction)(unsigned long))
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
  ETHERNET_SetMQTTReconnectCallBack(pFunction);  
#elif CONFIG_EC200S_USB_4G > 0
	Celluar_SetMQTTReconnectCallBack(pFunction);
#endif
}

/**
  * @brief  把MQTT状态函数赋值给底层，供底层状态变化时通知上层掉线或在线
  * @param  pFunction: 重连函数
  * @retval None
  */
void MQTT_StatusChangedFuncAssignFor(void (*pFunction)(unsigned long, bool))
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
  ETHERNET_SetMQTTStatusChangedCallBack(pFunction);
#elif CONFIG_EC200S_USB_4G > 0
	Celluar_SetMQTTStatusChangedCallBack(pFunction);
#endif
}

/**
  * @brief  清除对应接收队列
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval None
  */
void MQTT_ClearSeqQueue(unsigned long ulSocketID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
  return;	
#elif CONFIG_EC200S_USB_4G > 0
	Celluar_ClearSeqQueue(ulSocketID);
#endif
}

/**
  * @brief  获取队列数据
  * @param  ulSocketID: MQTT Socket 句柄
  * @param  pucBuffer: 队列接收缓存
  * @param  ulSize: 数据大小
  * @retval 返回接收到的长度
  */
long MQTT_RecvFromSeqQueue(unsigned long ulSocketID, unsigned char *pucBuffer, unsigned long ulSize)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
  return 0;
#elif CONFIG_EC200S_USB_4G > 0
	return Celluar_Recv(ulSocketID, pucBuffer, ulSize);
#else
  return 0;
#endif
}

/**
  * @brief  发送数据
  * @param  ulSocketID: MQTT Socket 句柄
  * @param  pucData: 发送缓存
  * @param  ulLen: 数据大小
  * @retval 返回是否发送成功MQTT_TRUE/MQTT_FALSE/MQTT_TIMEOUT
  */
MQTT_Status MQTT_SendData(unsigned long ulSocketID, unsigned char *pucData, unsigned long ulLen)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
    return MQTT_TRUE;
#elif CONFIG_EC200S_USB_4G > 0
	return (MQTT_Status)pFuncCelluar_Send(ulSocketID, pucData, ulLen);
#else
  return 0;
#endif
}

/**
  * @brief  获取当前对应Socket状态
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval MQTT_TRUE/MQTT_FALSE/MQTT_TIMEOUT
  */
MQTT_Status MQTT_GetSocketStatus(unsigned long ulSocketID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	return MQTT_TRUE;
#elif CONFIG_EC200S_USB_4G > 0
	return (MQTT_Status)pFuncCelluar_GetSocketStatus(ulSocketID);
#else
  return MQTT_TRUE;
#endif
}

/**
  * @brief  Socket TCP UDP等进行连接
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval MQTT_TRUE/MQTT_FALSE/MQTT_TIMEOUT
  */
MQTT_Status MQTT_SocketConnect(unsigned long ulSocketID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	return MQTT_TRUE;
#elif CONFIG_EC200S_USB_4G > 0
	return (MQTT_Status)pFuncCelluar_Connect(ulSocketID);
#else
  return MQTT_TRUE;
#endif
}

/**
  * @brief  断开Socket连接
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval MQTT_TRUE/MQTT_FALSE/MQTT_TIMEOUT
  */
MQTT_Status MQTT_SocketDisconnect(unsigned long ulSocketID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	return MQTT_TRUE;
#elif CONFIG_EC200S_USB_4G > 0
	return (MQTT_Status)pFuncCelluar_Disconnect(ulSocketID);
#else
  return MQTT_TRUE;  
#endif
}

/**
  * @brief  设置Socket状态
  * @param  ulSocketID: MQTT Socket 句柄
  * @retval None
  */
void MQTT_SetSocketStatus(unsigned long ulSocketID, bool bStatus)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
  ETHERNET_SetSocketStatus(ulSocketID, bStatus); 
#elif CONFIG_EC200S_USB_4G > 0
	Celluar_SetSocketStatus(ulSocketID, bStatus);
#endif
}

