/* Includes ------------------------------------------------------------------*/
#include "mqtts.h"
#include "mqtt_port.h"
#include "mqtt_interface.h"
#include <string.h>
#include "esp_log.h"

/* Extern variables ----------------------------------------------------------*/
/* Private macros ------------------------------------------------------------*/
#define MQTT_NAME_MAX_SIZE       48

/* Private types -------------------------------------------------------------*/
typedef struct
{
	char cClientID[MQTT_NAME_MAX_SIZE];
	char cUserName[MQTT_NAME_MAX_SIZE];
	char cPassword[MQTT_NAME_MAX_SIZE];
	unsigned char ucSendBuffer[MQTT_SEND_BUFFER_SIZE];
	unsigned char ucReadBuffer[MQTT_READ_BUFFER_SIZE];
	unsigned long ulKeepAliveTime;
	MQTTClient xClient;
	Network xNetWork;
	char cIP[4];
	int iport;
} MQTT;
/* Private variables ---------------------------------------------------------*/
static MQTT g_xMQTT[MQTT_MAX_SOCKET_COUNT] = {0};
static void (*pFuncMQTTConnectHandler)(unsigned long, bool);

/* Private functions ---------------------------------------------------------*/
static void MQTT_SetConnectStatus(unsigned long ulMQTTID, bool bStatus);
static bool MQTT_Reconnect(unsigned long ulMQTTID);

/**
 * @brief  MQTT Init
 * @param
 * @retval None
 */
long MQTT_ClientInit(char *pcClientID, char *pcHost, char *pcPort, char *pcUserName, char *pcPassword, unsigned long ulKeepAliveTime, unsigned long ulTimeout)
{
	long lMQTTID = -1;

	// if valid
	if (pcClientID == NULL || pcHost == NULL || pcPort == NULL || pcUserName == NULL || pcPassword == NULL)
		return lMQTTID;

	// Create mqtt id
	lMQTTID = MQTT_CreateSocket(MQTT_TCP_MODE_NAME, pcHost, pcPort);
	if (lMQTTID >= 0)
	{
		g_xMQTT[lMQTTID].xNetWork.my_socket = lMQTTID;
		// copy info
		strcpy(g_xMQTT[lMQTTID].cClientID, pcClientID);
		strcpy(g_xMQTT[lMQTTID].cUserName, pcUserName);
		strcpy(g_xMQTT[lMQTTID].cPassword, pcPassword);
		g_xMQTT[lMQTTID].ulKeepAliveTime = ulKeepAliveTime;
        NewNetwork(&g_xMQTT[lMQTTID].xNetWork, lMQTTID);	
		MQTTClientInit(&g_xMQTT[lMQTTID].xClient, &g_xMQTT[lMQTTID].xNetWork, ulTimeout, g_xMQTT[lMQTTID].ucSendBuffer, MQTT_SEND_BUFFER_SIZE, g_xMQTT[lMQTTID].ucReadBuffer, MQTT_READ_BUFFER_SIZE);
	}
	return lMQTTID;
}

/**
 * @brief  MQTT connect
 * @param
 * @retval None
 */
bool MQTT_Connect(unsigned long ulMQTTID, void (*pFuncConnectCallBack)(unsigned long, bool), char *addr, int port)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	int rc = 0;
#endif
	MQTTPacket_connectData xConnectData = MQTTPacket_connectData_initializer;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;
	pFuncMQTTConnectHandler = pFuncConnectCallBack;
	MQTT_ReconnectFuncAssignFor(MQTT_Reconnect);
	MQTT_StatusChangedFuncAssignFor(MQTT_SetConnectStatus);
	// Check if is already connected
	if ((MQTT_GetSocketStatus(ulMQTTID)) != MQTT_FALSE)
	{
		MQTT_SocketDisconnect(ulMQTTID);
	}
#if defined(MQTT_TASK)
	// Delete task
	if (g_xMQTT[ulMQTTID].xClient.thread.task != NULL)
	{
#if DEBUG > 0
		printf("Delete mqtt poll task.\r\n");
#endif
		MutexLock(&g_xMQTT[ulMQTTID].xClient.mutex);
		if (g_xMQTT[ulMQTTID].xClient.thread.task != NULL)
		{
			vTaskDelete(g_xMQTT[ulMQTTID].xClient.thread.task);
			g_xMQTT[ulMQTTID].xClient.thread.task = NULL;
		}
		MutexUnlock(&g_xMQTT[ulMQTTID].xClient.mutex);
	}
#endif
	if (MQTT_SocketConnect(ulMQTTID) == MQTT_TRUE)
	{
		xConnectData.clientID.cstring = g_xMQTT[ulMQTTID].cClientID;
		xConnectData.username.cstring = g_xMQTT[ulMQTTID].cUserName;
		xConnectData.password.cstring = g_xMQTT[ulMQTTID].cPassword;
		xConnectData.keepAliveInterval = g_xMQTT[ulMQTTID].ulKeepAliveTime;
		xConnectData.MQTTVersion = 3;
    #if CONFIG_W5100S_SPI_ETHERNET  > 0
		g_xMQTT[ulMQTTID].cIP[0] =addr[0];
		g_xMQTT[ulMQTTID].cIP[1] =addr[1];
		g_xMQTT[ulMQTTID].cIP[2] =addr[2];
		g_xMQTT[ulMQTTID].cIP[3] =addr[3];
		g_xMQTT[ulMQTTID].iport = port;
    	rc = mqtt_eth_network(ulMQTTID);
    #endif
		if(MQTTConnect(&g_xMQTT[ulMQTTID].xClient, &xConnectData) == 0)
		{
#if defined(MQTT_TASK)
			if (g_xMQTT[ulMQTTID].xClient.thread.task == NULL)
	  		{
				MQTTStartTask(&g_xMQTT[ulMQTTID].xClient);
			}
#endif    
			if (pFuncConnectCallBack != NULL)
			#if CONFIG_W5100S_SPI_ETHERNET > 0  
				pFuncConnectCallBack(ulMQTTID, rc);
			#elif CONFIG_EC200S_USB_4G > 0
				pFuncConnectCallBack(ulMQTTID, true);
			#endif
			return true;
		}
		else
		{
			// Disconnect
			MQTT_SocketDisconnect(ulMQTTID);
		}
	}

	if (pFuncConnectCallBack != NULL)
		pFuncConnectCallBack(ulMQTTID, false);
	return false;
}

/**
 * @brief  MQTT subscribe
 * @param
 * @retval None
 */
bool MQTT_Subscribe(unsigned long ulMQTTID, char *pcTopic, enum QoS eQoS, messageHandler pFuncHandler)
{
	if (pcTopic == NULL)
		return false;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	if (MQTTSubscribe(&g_xMQTT[ulMQTTID].xClient, pcTopic, eQoS, pFuncHandler) == 0)
	{
		return true;
	}

	return false;
}

/**
 * @brief  MQTT unsubscribe
 * @param
 * @retval None
 */
bool MQTT_Unsubscribe(unsigned long ulMQTTID, char *pcTopic)
{
	if (pcTopic == NULL)
		return false;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	if (MQTTUnsubscribe(&g_xMQTT[ulMQTTID].xClient, pcTopic) == 0)
	{
		return true;
	}

	return false;
}

/**
 * @brief  MQTT publish
 * @param
 * @retval None
 */
bool MQTT_Publish(unsigned long ulMQTTID, char *pcTopic, unsigned char *pucData, unsigned long ulDataSize,
				  bool bRetained, bool bDup, enum QoS eQoS)
{
	MQTTMessage xMessage;

	if (pcTopic == NULL || pucData == NULL)
		return false;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	xMessage.qos = eQoS;
	xMessage.retained = bRetained;
	xMessage.dup = bDup;
	xMessage.payload = pucData;
	xMessage.payloadlen = ulDataSize;

	if (MQTTPublish(&g_xMQTT[ulMQTTID].xClient, pcTopic, &xMessage) == 0)
	{
		return true;
	}

	return false;
}

bool MQTT_PublishFromExtMemory(unsigned long ulMQTTID, char *pcTopic, unsigned long ulDataSize, bool bRetained,
							   bool bDup, enum QoS eQoS, void (*pFuncExtMemoryRead)(unsigned long, unsigned char *, unsigned long), unsigned long ulMemoryStartAddress)
{
	MQTTMessage xMessage;

	if (pcTopic == NULL)
		return false;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	xMessage.qos = eQoS;
	xMessage.retained = bRetained;
	xMessage.dup = bDup;
	xMessage.payload = MQTT_SendBufferPointer(ulMQTTID);
	xMessage.payloadlen = ulDataSize;

	if (MQTTPublishFromExtMemory(&g_xMQTT[ulMQTTID].xClient, pcTopic, &xMessage, pFuncExtMemoryRead, ulMemoryStartAddress) == 0)
	{
		return true;
	}

	return false;
}

/**
 * @brief  MQTT change connect info
 * @param
 * @retval None
 */
bool MQTT_Disconnect(unsigned long ulMQTTID)
{
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	if (MQTTDisconnect(&g_xMQTT[ulMQTTID].xClient) == 0)
	{
		MQTT_ReleaseSocket(ulMQTTID);
		return true;
	}

	return false;
}

/**
 * @brief  MQTT get status
 * @param
 * @retval None
 */
bool MQTT_GetStatus(unsigned long ulMQTTID)
{
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;
	return (bool)g_xMQTT[ulMQTTID].xClient.isconnected;
}

/**
 * @brief  MQTT buffer pointer
 * @param
 * @retval pointer
 */
unsigned char *MQTT_SendBufferPointer(unsigned long ulMQTTID)
{
	return (g_xMQTT[ulMQTTID].ucSendBuffer + 128);
}

/**
 * @brief  MQTT return rest send buffer size
 * @param
 * @retval buffer size
 */
unsigned long MQTT_GetRestSendBufferSize(unsigned long ulMQTTID)
{
	return (MQTT_SEND_BUFFER_SIZE - 128);
}

/**
 * @brief  MQTT change timeout
 * @param
 * @retval None
 */
void MQTT_ChangeTimeout(unsigned long ulMQTTID, unsigned long ulTimeout)
{
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return;

	g_xMQTT[ulMQTTID].xClient.command_timeout_ms = ulTimeout;
}

/**
 * @brief  MQTT set status
 * @param
 * @retval None
 */
static void MQTT_SetConnectStatus(unsigned long ulMQTTID, bool bStatus)
{
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return;
    
	if (g_xMQTT[ulMQTTID].xClient.isconnected == true && bStatus == false)
	{
#if DEBUG > 0
		printf("MQTT socket closed.\r\n");
#endif
	}

	g_xMQTT[ulMQTTID].xClient.isconnected = bStatus;
}

/**
 * @brief  MQTT reconnect
 * @param
 * @retval None
 */
static bool MQTT_Reconnect(unsigned long ulMQTTID)
{
#if CONFIG_W5100S_SPI_ETHERNET > 0
	int rc = 0;
#endif
	MQTTPacket_connectData xConnectData = MQTTPacket_connectData_initializer;
	if (ulMQTTID >= MQTT_MAX_SOCKET_COUNT)
		return false;

	// clear
	MQTT_ClearSeqQueue(ulMQTTID);

	// Delete task
	if (g_xMQTT[ulMQTTID].xClient.thread.task != NULL)
	{
#if DEBUG > 0
		printf("Delete mqtt poll task.\r\n");
#endif
#if defined(MQTT_TASK)
		MutexLock(&g_xMQTT[ulMQTTID].xClient.mutex);
		if (g_xMQTT[ulMQTTID].xClient.thread.task != NULL)
		{
			vTaskDelete(g_xMQTT[ulMQTTID].xClient.thread.task);
			g_xMQTT[ulMQTTID].xClient.thread.task = NULL;
		}
		MutexUnlock(&g_xMQTT[ulMQTTID].xClient.mutex);
#endif
	}

	xConnectData.clientID.cstring = g_xMQTT[ulMQTTID].cClientID;
	xConnectData.username.cstring = g_xMQTT[ulMQTTID].cUserName;
	xConnectData.password.cstring = g_xMQTT[ulMQTTID].cPassword;
	xConnectData.keepAliveInterval = g_xMQTT[ulMQTTID].ulKeepAliveTime;
	xConnectData.MQTTVersion = 3;
#if CONFIG_W5100S_SPI_ETHERNET > 0 	
    rc = mqtt_eth_network(ulMQTTID);
#endif
	if (MQTTConnect(&g_xMQTT[ulMQTTID].xClient, &xConnectData) == 0)
	{
#if defined(MQTT_TASK)
		if (g_xMQTT[ulMQTTID].xClient.thread.task == NULL)
		{
			MQTTStartTask(&g_xMQTT[ulMQTTID].xClient);
		}
#endif
		#if CONFIG_W5100S_SPI_ETHERNET  > 0
			pFuncMQTTConnectHandler(ulMQTTID, rc);
		#elif CONFIG_EC200S_USB_4G > 0
			pFuncMQTTConnectHandler(ulMQTTID, true);   
		#endif

		return true;
	}
	else
	{
		// Disconnect
		MQTT_SocketDisconnect(ulMQTTID);

		if (pFuncMQTTConnectHandler != NULL)
			pFuncMQTTConnectHandler(ulMQTTID, false);
	}

	return false;
}

#if CONFIG_W5100S_SPI_ETHERNET  > 0
bool mqtt_eth_network(unsigned long ulMQTTID)
{
	int rc = 0;
	rc = NetworkConnect(&g_xMQTT[ulMQTTID].xNetWork, g_xMQTT[ulMQTTID].cIP,  g_xMQTT[ulMQTTID].iport);
	if(rc == 1)
		return true;
    return false;
}
#endif


